@@ -16,7 +16,7 @@ import unittest
 import gnupg
 
 __author__ = "Vinay Sajip"
-__date__  = "$11-Mar-2013 23:48:26$"
+__date__  = "$30-Aug-2013 18:10:57$"
 
 ALL_TESTS = True
 
@@ -184,6 +184,7 @@ class GPGTestCase(unittest.TestCase):
                               '(dummy comment) <test.name@example.com>')
 
     def test_key_generation_with_escapes(self):
+        "Test that key generation handles escape characters"
         cmd = self.gpg.gen_key_input(name_comment='Funny chars: '
                                                   '\\r\\n\\f\\v\\0\\b',
                                      name_real='Test Name',
@@ -201,15 +202,14 @@ class GPGTestCase(unittest.TestCase):
     def test_key_generation_with_empty_value(self):
         "Test that key generation handles empty values"
         params = {
-            'key_type': 'RSA',
-            'key_length': 1024,
-            'name_comment': ' ', # Not added, so default will appear
+            'key_type': ' ',
+            'key_length': 2048,
         }
         cmd = self.gpg.gen_key_input(**params)
-        self.assertTrue('\nName-Comment: Generated by gnupg.py\n' in cmd)
-        params['name_comment'] = 'A'
+        self.assertTrue('Key-Type: RSA\n' in cmd)
+        params['key_type'] = 'DSA'
         cmd = self.gpg.gen_key_input(**params)
-        self.assertTrue('\nName-Comment: A\n' in cmd)
+        self.assertTrue('Key-Type: DSA\n' in cmd)
         
     def test_list_keys_after_generation(self):
         "Test that after key generation, the generated key is available"
@@ -265,6 +265,12 @@ class GPGTestCase(unittest.TestCase):
         edata = str(gpg.encrypt(data, None, passphrase='bbrown', symmetric=True))
         ddata = gpg.decrypt(edata, passphrase='bbrown')
         self.assertEqual(data, str(ddata))
+        # Test symmetric encryption with non-default cipher
+        data = "chippy was here"
+        edata = str(gpg.encrypt(data, None, passphrase='bbrown',
+                    symmetric='AES256'))
+        ddata = gpg.decrypt(edata, passphrase='bbrown')
+        self.assertEqual(data, str(ddata))
 
     def test_import_and_export(self):
         "Test that key import and export works"
@@ -446,6 +452,15 @@ class GPGTestCase(unittest.TestCase):
                     os.remove(fn)
         logger.debug("test_file_encryption_and_decryption ends")
 
+    def test_search_keys(self):
+        "Test that searching for keys works"
+        r = self.gpg.search_keys('<vinay_sajip@hotmail.com>')
+        self.assertTrue(r)
+        self.assertTrue('Vinay Sajip <vinay_sajip@hotmail.com>' in r[0]['uids'])
+        r = self.gpg.search_keys('92905378')
+        self.assertTrue(r)
+        self.assertTrue('Vinay Sajip <vinay_sajip@hotmail.com>' in r[0]['uids'])
+
 
 TEST_GROUPS = {
     'sign' : set(['test_signature_verification']),
@@ -456,7 +471,8 @@ TEST_GROUPS = {
                  'test_key_generation_with_invalid_key_type',
                  'test_key_generation_with_escapes',
                  'test_key_generation_with_empty_value',
-                 'test_key_generation_with_colons']),
+                 'test_key_generation_with_colons',
+                 'test_search_keys']),
     'import' : set(['test_import_only']),
     'basic' : set(['test_environment', 'test_list_keys_initial',
                    'test_nogpg', 'test_make_args']),
